from .core import (
    CRUDRouter,
    MongoModel,
    MongoObjectId,
    CRUDLookup,
)

__all__ = [
    "CRUDRouter",
    "MongoModel",
    "MongoObjectId",
    "CRUDLookup",
]

__version__ = "0.0.4"
__author__ = "Pierre DUVEAU"
__credits__ = ["Pierre DUVEAU", "Adam Watkins"]
